/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ftp.subscriber;

import java.util.*;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.NullSubProgressMonitor;
import org.eclipse.team.internal.ftp.*;
import org.eclipse.team.internal.ftp.client.FTPClient;
import org.eclipse.team.internal.ftp.client.FTPDirectoryEntry;
import org.eclipse.team.internal.target.subscriber.PutOperation;
import org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider;

/**
 * FTP specific put operation
 */
public class FTPPutOperation extends PutOperation {

	private List changed = new ArrayList();
	
	public FTPPutOperation(
		IResource[] resources,
		int depth,
		boolean overwriteRemoteChanges) {
		super(FTPPlugin.DEPLOYMENT_PROVIDER_ID, resources, depth, overwriteRemoteChanges);
	}

	protected void needsRemoteIdentifier(TargetDeploymentProvider provider, IResource localResource, IProgressMonitor monitor) throws TeamException {
		if (localResource.getType() == IResource.FILE) {
			changed.add(localResource);
		} else {
			((FTPSynchronizer)FTPPlugin.getPlugin().getSubscriber().getSynchronizer()).setBaseBytes(localResource, FTPSubscriberResource.FOLDER_BYTES);
		}
	}
	
	private void provideRemoteIdentifiers(TargetDeploymentProvider provider, IProgressMonitor progress) throws TeamException {
		// Get the remote identifiers for any resources that require it
		if ( ! changed.isEmpty()) {
			try {
				// group the resources by parent directory
				final Map resourcesByParent = new HashMap();
				for (Iterator iter = changed.iterator(); iter.hasNext();) {
					IResource resource = (IResource) iter.next();
					List children = (List) resourcesByParent.get(resource.getParent());
					if (children == null) {
						children = new ArrayList();
						resourcesByParent.put(resource.getParent(), children);
					}
					children.add(resource);
				}
				// Update the base bytes for all the resources
				IProgressMonitor noProgress = new NullSubProgressMonitor(progress);
				for (Iterator iter = resourcesByParent.keySet().iterator(); iter.hasNext();) {
					IContainer parent = (IContainer) iter.next();
					List children = (List) resourcesByParent.get(parent);
					FTPDirectoryEntry entries[] = getEntries(provider, parent, noProgress);
					for (Iterator iterator = children.iterator(); iterator.hasNext();) {
						IResource resource = (IResource) iterator.next();
						FTPDirectoryEntry entry = findEntry(resource.getName(), entries);
						if (entry == null) {
							throw new FTPException(Policy.bind("FTPPutOperation.0", resource.getFullPath().toString())); //$NON-NLS-1$
						}
						getSynchronizer(provider).setBaseBytes(resource, FTPSubscriberResource.getSyncBytes(entry));
					}
				}
			} finally {
				// remove the processed nodes from the list
				changed.clear();
			}
		}
	}
	
	private FTPDirectoryEntry[] getEntries(TargetDeploymentProvider provider, IContainer parent, IProgressMonitor monitor) throws FTPException, TeamException {
		FTPDeploymentProvider ftp = (FTPDeploymentProvider)provider;
		return getClient(ftp).listFiles(ftp.getRelativePath(parent).toString(), monitor);
	}

	private FTPDirectoryEntry findEntry(String name, FTPDirectoryEntry[] entries) {
		for (int i = 0; i < entries.length; i++) {
			FTPDirectoryEntry entry = entries[i];
			if (entry.getName().equals(name)) {
				return entry;
			}
		}
		return null;
	}
	
	protected String getRemoteRoot(FTPDeploymentProvider provider) {
		return provider.getHostRelativePath();
	}
	
	protected String getProviderRelativePath(FTPDeploymentProvider provider, IResource resource) throws TeamException {
		return provider.getRelativePath(resource).toString();
	}
	
	public FTPClient getClient(FTPDeploymentProvider provider) {
		return provider.getOpenClient();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.ftp.deployment.FTPOperation#execute(org.eclipse.team.core.target.TargetDeploymentProvider, org.eclipse.core.resources.IResource[], org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void execute(
		TargetDeploymentProvider provider,
		IResource[] resources,
		IProgressMonitor monitor)
		throws TeamException {
		
		monitor.beginTask(null, 100);
		super.execute(provider, resources, Policy.subMonitorFor(monitor, 80));
		provideRemoteIdentifiers(provider, Policy.subMonitorFor(monitor, 20));
		monitor.done();
	}

}
