/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ftp.subscriber;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.core.Team;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.CachedResourceVariant;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.internal.core.streams.ProgressMonitorInputStream;
import org.eclipse.team.internal.core.target.UrlUtil;
import org.eclipse.team.internal.ftp.FTPException;
import org.eclipse.team.internal.ftp.FTPPlugin;
import org.eclipse.team.internal.ftp.Policy;
import org.eclipse.team.internal.ftp.client.Constants;
import org.eclipse.team.internal.ftp.client.FTPClient;
import org.eclipse.team.internal.ftp.client.FTPDirectoryEntry;
import org.eclipse.team.internal.ftp.client.IFTPRunnable;
import org.eclipse.team.internal.ftp.client.IFTPRunnableContext;

public class FTPSubscriberResource extends CachedResourceVariant {

	public static final byte[] FOLDER_BYTES = "D0:0".getBytes(); //$NON-NLS-1$
	
	private IFTPRunnableContext context;
	private URL url;
	private boolean isContainer;
	private FTPDirectoryEntry entry;

	private static final int MAX_TRANSFER_SIZE = 32788;
	
	public static byte[] getSyncBytes(FTPDirectoryEntry entry) {
		// Return enough to rebuild the handle
		long modTime = entry.getModTime().getTime();
		long size = entry.getSize();
		String type = entry.hasFileSemantics() ? "F" : "D"; //$NON-NLS-1$ //$NON-NLS-2$
		return (type + new Long(modTime).toString() + ":" + new Long(size).toString()).getBytes(); //$NON-NLS-1$
	}
	
	/**
	 * Create a resource from the given parent and entry
	 * @param remote
	 * @param entry2
	 * @return
	 */
	public static FTPSubscriberResource create(FTPSubscriberResource parent, FTPDirectoryEntry entry) throws FTPException {
		try {
			return new FTPSubscriberResource(
					parent.getContext(), 
					UrlUtil.concat(parent.getUrl().toExternalForm(), new Path(entry.getName())),
					entry.hasDirectorySemantics(),
					entry);
		} catch (MalformedURLException e) {
			throw new FTPException(Policy.bind("FTPSubscriberResource.8", parent.getUrl().toExternalForm(),  entry.getName()), e); //$NON-NLS-1$
		}
	}
	
	/**
	 * @param ftpProvider
	 * @param resource
	 * @param entry2
	 * @return
	 */
	public static IResourceVariant create(FTPDeploymentProvider ftpProvider, IResource resource, FTPDirectoryEntry entry) throws FTPException {
		try {
			return new FTPSubscriberResource(ftpProvider, ftpProvider.getUrl(resource), resource.getType() != IResource.FILE, entry);
		} catch (MalformedURLException e) {
			throw new FTPException(Policy.bind("FTPSubscriberResource.9", resource.getFullPath().toString()), e); //$NON-NLS-1$
		}
	}
	
	public static FTPSubscriberResource create(IFTPRunnableContext context, URL url, byte[] syncBytes) {
		boolean isContainer = (syncBytes[0] == 'D');
		String id = new String(syncBytes);
		int colon = id.indexOf(":"); //$NON-NLS-1$
		long modTime = Long.parseLong(id.substring(1, colon));
		long size = Long.parseLong(id.substring(colon+1));
		IPath path = new Path(url.getPath());
		String name;
		if (path.isEmpty()) {
			name = "."; //$NON-NLS-1$
		} else {
			name = new Path(url.getPath()).lastSegment();
		}
		FTPDirectoryEntry entry = new FTPDirectoryEntry(name, isContainer, !isContainer, size, new Date(modTime));
		return new FTPSubscriberResource(context, url, isContainer, entry);
	}
	
	public FTPSubscriberResource(IFTPRunnableContext context, URL url, boolean isContainer, FTPDirectoryEntry entry) {
		this.entry = entry;
		this.isContainer = isContainer;
		this.url = url;
		this.context = context;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ISubscriberResource#getName()
	 */
	public String getName() {
		return getEntry().getName();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ISubscriberResource#isContainer()
	 */
	public boolean isContainer() {
		return isContainer;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.sync.IRemoteResource#getContents(org.eclipse.core.runtime.IProgressMonitor)
	 */
	private InputStream getContents(IProgressMonitor progress) throws TeamException {
		final InputStream result[] = new InputStream[] { null };
		context.run(new IFTPRunnable() {
			public void run(IProgressMonitor monitor) throws TeamException {
				monitor.beginTask(null, 100);
				try {
					// get the size and only transfer if it is under the max transfer size
					long size = getEntry().getSize();
					if (size > MAX_TRANSFER_SIZE) {
						result[0] = new ByteArrayInputStream(Policy.bind("FTPRemoteTargetResource.remoteFileTooBig").getBytes()); //$NON-NLS-1$
						return;
					}
					final String title = Policy.bind("FTPClient.getFile", context.getRelativePath(getUrl()).toString()); //$NON-NLS-1$
					monitor.subTask(title);
					InputStream in = getClient().getContents(context.getRelativePath(getUrl()).toString(), isBinary(), 0L, Policy.subMonitorFor(monitor, 10));
					// setup progress monitoring
					IProgressMonitor subMonitor = Policy.subMonitorFor(monitor, 80);
					try {
						subMonitor.beginTask(null, (int)size);
						subMonitor.subTask(Policy.bind("FTPClient.transferNoSize", title)); //$NON-NLS-1$
						in = new ProgressMonitorInputStream(in, 0, Constants.TRANSFER_PROGRESS_INCREMENT, subMonitor) {
							long previousBytesRead = 0;
							protected void updateMonitor(long bytesRead, long bytesTotal, IProgressMonitor monitor) {
								if (bytesRead == 0) return;
								long worked = bytesRead  - previousBytesRead;
								previousBytesRead = bytesRead;
								monitor.worked((int)worked);
								monitor.subTask(Policy.bind("FTPClient.transferUnknownSize", //$NON-NLS-1$
										new Object[] { title, Long.toString(bytesRead >> 10) }));
							}
						};
						
						// Transfer the contents
						byte[] buffer = new byte[1024];
						ByteArrayOutputStream out = new ByteArrayOutputStream();
						int read;
						try {
							try {
								while ((read = in.read(buffer)) >= 0) {
									out.write(buffer, 0, read);
								}
							} finally {
								out.close();
							}
						} catch (IOException e) {
							throw FTPPlugin.wrapException(e);
						} finally {
							try {
								in.close();
							} catch (IOException e) {
								FTPPlugin.log(FTPPlugin.wrapException(e));
							}
						}
						result[0] = new ByteArrayInputStream(out.toByteArray());
					} finally {
						subMonitor.done();
					}
				} finally {
					monitor.done();
				}
			}
		}, Policy.monitorFor(progress));
		return result[0];
	}
	
	/* private */ boolean isBinary() {
		return Team.getType(new IStorage() {
			public InputStream getContents() throws CoreException {
				// use empty contents when trying to determine the file type
				return new ByteArrayInputStream(new byte[0]);
			}
			public IPath getFullPath() {
				return new Path(getUrl().getPath());
			}
			public String getName() {
				return FTPSubscriberResource.this.getName();
			}
			public boolean isReadOnly() {
				return true;
			}
			public Object getAdapter(Class adapter) {
				return null;
			}
		}) != Team.TEXT;
	}
	
	/* private */ FTPClient getClient() {
		return getContext().getOpenClient();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ISubscriberResource#getContentIdentifier()
	 */
	public String getContentIdentifier() {
		return getEntry().getModTime().toString();
	}

	public IFTPRunnableContext getContext() {
		return context;
	}

	public FTPDirectoryEntry getEntry() {
		return entry;
	}

	public URL getUrl() {
		return url;
	}

	public byte[] asBytes() {
		if (isContainer) {
			return FOLDER_BYTES;
		}
		return getSyncBytes(getEntry());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.synchronize.ResourceVariant#fetchContents(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void fetchContents(IProgressMonitor monitor) throws TeamException {
		setContents(getContents(Policy.subMonitorFor(monitor, 75)), Policy.subMonitorFor(monitor, 25));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.synchronize.ResourceVariant#getUniquePath()
	 */
	public String getCachePath() {
		return getUrl().toExternalForm() + ' ' + getContentIdentifier();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.synchronize.ResourceVariant#getCacheId()
	 */
	protected String getCacheId() {
		return FTPPlugin.ID;
	}

}
