/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.target.subscriber;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.variants.*;
import org.eclipse.team.internal.core.target.DeploymentProvider;
import org.eclipse.team.internal.core.target.IDeploymentProviderManager;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;

/**
 * A generalized target subscriber
 */
public abstract class TargetSubscriber extends ThreeWaySubscriber implements ITargetOperations {

	protected TargetSubscriber(ThreeWaySynchronizer synchronizer) {
		super(synchronizer);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.TeamSubscriber#isSupervised(org.eclipse.core.resources.IResource)
	 */
	public boolean isSupervised(IResource resource) throws TeamException {
		return getDeploymentManager().getMappedTo(resource, getDeploymentProviderId());
	}

	/**
	 * Return the id of the deployment provider associated with this subscriber
	 */
	protected abstract String getDeploymentProviderId();

	/**
	 * Helper method which returns the Team deployment manager.
	 */
	protected IDeploymentProviderManager getDeploymentManager() {
		return TeamWebDavPlugin.getDeploymentManager();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.TeamSubscriber#roots()
	 */
	public IResource[] roots() {
		return getDeploymentManager().getDeploymentProviderRoots(getDeploymentProviderId());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ITeamOperations#get(org.eclipse.core.resources.IResource[], int, boolean, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus get(IResource[] resources, int depth, boolean overwriteLocalChanges, IProgressMonitor progress) throws TeamException {
		new GetOperation(getDeploymentProviderId(), resources, depth, overwriteLocalChanges).run(progress);
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ITeamOperations#put(org.eclipse.core.resources.IResource[], int, boolean, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus put(IResource[] resources, int depth, boolean overwriteRemoteChanges, IProgressMonitor progress) throws TeamException {
		getPutOperation(resources, depth, overwriteRemoteChanges).run(progress);
		return Status.OK_STATUS;
	}

	protected PutOperation getPutOperation(IResource[] resources, int depth, boolean overwriteRemoteChanges) {
		return new PutOperation(getDeploymentProviderId(), resources, depth, overwriteRemoteChanges);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.TeamSubscriber#getTeamOperations()
	 */
	public ITargetOperations getTeamOperations() {
		return this;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.helpers.SyncTreeSubscriber#getSyncInfo(org.eclipse.core.resources.IResource, org.eclipse.team.core.subscribers.ISubscriberResource, org.eclipse.team.core.subscribers.ISubscriberResource, org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected SyncInfo getSyncInfo(
		IResource local,
		IResourceVariant base,
		IResourceVariant remote)
		throws TeamException {

		TargetSyncInfo info = new TargetSyncInfo(local, base, remote, this);
		info.init();
		return info;
	}

	public void rootAdded(IContainer container) {
		handleRootChanged(container, true);
	}

	public void rootRemoved(IContainer container) {
		handleRootChanged(container, false);
	}
	
	public IResourceVariant getResourceVariant(IResource resource, byte[] syncBytes) throws TeamException {
		if (syncBytes == null) return null;
		DeploymentProvider[] providers = TeamWebDavPlugin.getDeploymentManager().getMappings(resource, getDeploymentProviderId());
		if (providers.length == 0 
				|| ! (providers[0] instanceof TargetDeploymentProvider)) {
			return null;
		}
		TargetDeploymentProvider target = (TargetDeploymentProvider)providers[0];
		return target.getResourceVariant(resource, syncBytes);
	}
}
