/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ui.target.subscriber;

import java.lang.reflect.InvocationTargetException;
import java.util.ResourceBundle;

import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.synchronize.FastSyncInfoFilter;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.synchronize.SyncInfoSet;
import org.eclipse.team.core.synchronize.FastSyncInfoFilter.SyncInfoDirectionFilter;
import org.eclipse.team.internal.target.subscriber.ITargetOperations;
import org.eclipse.team.internal.ui.Utils;
import org.eclipse.team.internal.webdav.core.Policy;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.SynchronizeModelAction;
import org.eclipse.team.ui.synchronize.SynchronizeModelOperation;

/**
 * This class defines an action that uses the get operation of ITeamOperation.
 */
public abstract class GetAction extends SynchronizeModelAction {

	protected GetAction(ISynchronizePageConfiguration configuration) {
		super(null, configuration);
		Utils.initAction(this, getBundleKeyPrefix(), Policy.getBundle());
	}

	public GetAction(ISynchronizePageConfiguration configuration, ISelectionProvider provider, String string, ResourceBundle bundle) {
		super(null, configuration, provider);
		Utils.initAction(this, string, bundle);
	}

	/**
	 * Return the key to the action text in the resource bundle.
	 * The default is the class name followed by a dot (.).
	 * @return the bundle key prefix
	 */
	protected String getBundleKeyPrefix() {
		String name = getClass().getName();
		int lastDot = name.lastIndexOf("."); //$NON-NLS-1$
		if (lastDot == -1) {
			return name;
		}
		return name.substring(lastDot + 1)  + "."; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.ui.actions.SubscriberAction#getSubscriberOperation(org.eclipse.compare.structuremergeviewer.IDiffElement[])
	 */
	protected SynchronizeModelOperation getSubscriberOperation(ISynchronizePageConfiguration configuration, IDiffElement[] elements) {
		return new SynchronizeModelOperation(configuration, elements) {
			protected String getJobName() {
				return GetAction.this.getJobName(getSyncInfoSet());
			}
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try {
					SyncInfoSet syncSet = getSyncInfoSet();
					if (!promptForConflictHandling(getShell(), syncSet)) return;
					ITargetOperations operations = getTargetOperations();
					IStatus status = operations.get(syncSet.getResources(), IResource.DEPTH_ZERO, getLocalOverwrite(), monitor);
					throw new TeamException(status);
				} catch (TeamException e) {
					throw new InvocationTargetException(e);
				}
			}
			protected boolean canRunAsJob() {
				return true;
			}
		};
	}

	protected boolean getLocalOverwrite() {
		return false;
	}

	protected abstract String getJobName(SyncInfoSet syncSet);
	
	protected abstract ITargetOperations getTargetOperations();
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.ui.sync.SubscriberAction#getSyncInfoFilter()
	 */
	protected FastSyncInfoFilter getSyncInfoFilter() {
		return new SyncInfoDirectionFilter(new int[] {SyncInfo.INCOMING});
	}
	
	/**
	 * Prompts the user to determine how conflicting changes should be handled.
	 * Note: This method is designed to be overridden by test cases.
	 * @return 0 to sync conflicts, 1 to sync all non-conflicts, 2 to cancel
	 */
	protected int promptForConflicts(Shell shell, SyncInfoSet syncSet) {
		String[] buttons = new String[] {IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL, IDialogConstants.CANCEL_LABEL};
		String title = "Confirm Overwrite"; //$NON-NLS-1$
		String question = "You have changes that conflict with the server. Overwrite those changes?"; //$NON-NLS-1$
		String[] tips = new String[] {
				Policy.bind("GetAction.2"),  //$NON-NLS-1$
				Policy.bind("GetAction.3"), //$NON-NLS-1$
				Policy.bind("GetAction.4") //$NON-NLS-1$
		};
		final ToolTipMessageDialog dialog = new ToolTipMessageDialog(shell, title, null, question, MessageDialog.QUESTION, buttons, tips, 0);
		shell.getDisplay().syncExec(new Runnable() {
			public void run() {
				dialog.open();
			}
		});
		return dialog.getReturnCode();
	}
	
	protected boolean promptForConflictHandling(Shell shell, SyncInfoSet syncSet) {
		// If there is a conflict in the syncSet, we need to prompt the user before proceeding.
		if (syncSet.hasConflicts() || syncSet.hasOutgoingChanges()) {
			switch (promptForConflicts(shell, syncSet)) {
			case 0:
				// Yes, synchronize conflicts as well
				break;
			case 1:
				// No, stop here
				syncSet.removeConflictingNodes();
				syncSet.removeOutgoingNodes();
				break;
			case 2:
			default:
				// Cancel
				return getLocalOverwrite();
			}	
		}
		return true;
	}
}
