/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.webdav.subscriber;

import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.CachedResourceVariant;
import org.eclipse.team.internal.core.streams.PollingInputStream;
import org.eclipse.team.internal.core.streams.ProgressMonitorInputStream;
import org.eclipse.team.internal.core.target.ITeamStatusConstants;
import org.eclipse.team.internal.core.target.Site;
import org.eclipse.team.internal.core.target.UrlUtil;
import org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider;
import org.eclipse.team.internal.webdav.core.Policy;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;
import org.eclipse.team.internal.webdav.core.WebDavSite;
import org.eclipse.webdav.IResponse;
import org.eclipse.webdav.client.AbstractResourceHandle;
import org.eclipse.webdav.client.PropertyStatus;
import org.eclipse.webdav.dom.ElementEditor;
import org.eclipse.webdav.internal.kernel.ClientException;
import org.eclipse.webdav.internal.kernel.DAVException;
import org.eclipse.webdav.internal.kernel.WebDAVPropertyNames;
import org.w3c.dom.Element;

public class WebDavSubscriberResource extends CachedResourceVariant {

	/*
	 * This is a handle to the remote DAV element
	 */
	private AbstractResourceHandle remoteResource;
	private boolean isContainer;
	private byte[] syncBytes;
	private String contentIdentifier;
	
	public static WebDavSubscriberResource create(AbstractResourceHandle remote, boolean isContainer) throws TeamException {
		// TODO:may be fecthing sync bytes for no reason
		return new WebDavSubscriberResource(remote, isContainer, getRemoteSyncBytes(remote));
	}
	
	public static byte[] getRemoteSyncBytes(AbstractResourceHandle remote) throws TeamException {
		try {
			PropertyStatus prop =
				remote.getProperty(WebDAVPropertyNames.DAV_GET_E_TAG);
			
			if (prop.getStatusCode() == IResponse.SC_NOT_FOUND) {
				// some webdav implementors allow collection nodes to not contain
				// an etag. In this case we assume that no version info is
				// managed in that webdav server and so we make this 
				// resource either exist or not, by using a virtual etag of
				// the uri. If the resource does not exit, we 
				// will go for the default catch rule below, 
				// throwing a TeamException
				if(remote.exists())
					return remote.getLocator().getResourceURL().toString().getBytes();
				
			}
			if (prop.getStatusCode() != IResponse.SC_OK)
				throw new TeamException(ITeamStatusConstants.IO_FAILED_STATUS);
			Element value = prop.getProperty();
			return ElementEditor.getFirstText(value).getBytes(); //$NON-NLS-1$
		} catch (DAVException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		}
	}
	
	public WebDavSubscriberResource(AbstractResourceHandle remote, boolean isContainer, byte[] syncBytes) {
		this.remoteResource = remote;
		this.isContainer = isContainer;
		this.syncBytes = syncBytes;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.core.target.TargetSubscriberResource#getSyncBytes()
	 */
	public byte[] asBytes() {
		return syncBytes;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ISubscriberResource#getName()
	 */
	public String getName() {
		try {
			// The DAV remote resource's location can have the following forms:
			// http://www.server.com/
			// http://www.server.com/dav/auth/
			URL url = new URL(remoteResource.getLocator().getResourceURL());
			Path path = new Path(url.getFile());
			
			// if the remote resource is at the root of the remote site then
			// return the root path
			if(path.isRoot() || path.isEmpty()) {
				return path.toString();				
				// otherwise, if the resource is not at the root return the last 
				// segment of the location as its name
			} else {
				return path.lastSegment().toString();
			}
		} catch (MalformedURLException e) {
			TeamWebDavPlugin.log(IStatus.ERROR, e.getMessage(), e);
			return Policy.bind("DavRemoteTargetResource.errorName"); // $NON-NLS-1$ //$NON-NLS-1$
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ISubscriberResource#isContainer()
	 */
	public boolean isContainer() {
		return isContainer;
	}

	private InputStream getContents(IProgressMonitor monitor) throws TeamException {
		try {
			// number of bytes expected
			final int size = internalGetSize() >> 10;
			final String name = UrlUtil.toTruncatedPath(getResourceUrl(), 3);
			
			// timeout value for this site
			int timeout = getWebDavSite().getTimeout();
			
			final String prefix = Policy.bind("DavRemoteTargetResource.download", name); //$NON-NLS-1$
			monitor.subTask(Policy.bind("DavRemoteTargetResource.transferNoSize", prefix)); //$NON-NLS-1$
			
			// wrap the dav input stream with an input stream that allows progress
			// and cancellation.
			InputStream in = new ProgressMonitorInputStream(
					new PollingInputStream(remoteResource.getContent(), 
							timeout,
							monitor), 
							0, 
							WebDavDeploymentProvider.TRANSFER_PROGRESS_INCREMENT, 
							monitor) {
				// perform updates to the progress message, showing the number of bytes transfered
				protected void updateMonitor(long bytesRead, long bytesTotal, IProgressMonitor monitor) {
					if (bytesRead == 0) return;
					monitor.subTask(Policy.bind("DavRemoteTargetResource.transfer", //$NON-NLS-1$
							new Object[] { prefix,
							Long.toString(bytesRead >> 10),
							Integer.toString(size) }));
				}
			};																			
			
			return in;
		} catch (ClientException exception) {
			if (exception.getStatusCode() == IResponse.SC_NOT_FOUND)
				throw new TeamException(ITeamStatusConstants.NO_REMOTE_RESOURCE_STATUS);
			throw TeamWebDavPlugin.wrapException(exception);
		} catch (DAVException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		}
	}
	
	private int internalGetSize() throws TeamException {
		try {
			PropertyStatus prop =
				remoteResource.getProperty(WebDAVPropertyNames.DAV_GET_CONTENT_LENGTH);
			if (prop.getStatusCode() != IResponse.SC_OK)
				throw new TeamException(ITeamStatusConstants.IO_FAILED_STATUS);
			Element value = prop.getProperty();
			return new Integer(ElementEditor.getFirstText(value)).intValue();
		} catch (DAVException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		}
	}
	
	protected String getResourceUrl() {
		return remoteResource.getLocator().getResourceURL();
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.subscribers.ISubscriberResource#getContentIdentifier()
	 */
	public String getContentIdentifier() {
		if (contentIdentifier == null) {
			contentIdentifier = fetchContentIdentifier();
		}
		return contentIdentifier;
	}
	
	public String fetchContentIdentifier() {
		try {
			PropertyStatus prop =
				remoteResource.getProperty(WebDAVPropertyNames.DAV_GET_LAST_MODIFIED);
			
			if (prop.getStatusCode() == IResponse.SC_NOT_FOUND) {
				// some webdav implementors allow collection nodes to not contain
				// an etag. In this case we assume that no version info is
				// managed in that webdav server and so we make this 
				// resource either exist or not, by using a virtual etag of
				// the uri. If the resource does not exit, we 
				// will go for the default catch rule below, 
				// throwing a TeamException
				if(remoteResource.exists())
					return remoteResource.getLocator().getResourceURL().toString();
				
			}
			
			if (prop.getStatusCode() != IResponse.SC_OK)
				return Policy.bind("WebDavSubscriberResource.5"); //$NON-NLS-1$
			Element value = prop.getProperty();
			return ElementEditor.getFirstText(value);
		} catch (DAVException exception) {
			TeamWebDavPlugin.log(new Status(IStatus.ERROR, TeamWebDavPlugin.getPlugin().getDescriptor().getUniqueIdentifier(), 0, exception.getMessage(), exception));
			return Policy.bind("WebDavSubscriberResource.6"); //$NON-NLS-1$
		}
	}

	protected WebDavSite getWebDavSite() throws TeamException {
		try {
			Site site = TargetDeploymentProvider.getSite(new URL(getResourceUrl()));
			if (site instanceof WebDavSite) {
				return (WebDavSite)site;
			}
		} catch (MalformedURLException e) {
			throw TeamWebDavPlugin.wrapException(e);
		}
		throw new TeamException(Policy.bind("WebDavSubscriberResource.4", getResourceUrl()), null); //$NON-NLS-1$
	}

	public AbstractResourceHandle getDavHandle() {
		return remoteResource;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.synchronize.ResourceVariant#fetchContents(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void fetchContents(IProgressMonitor monitor) throws TeamException {
		getContentIdentifier();
		setContents(getContents(Policy.subMonitorFor(monitor, 75)), Policy.subMonitorFor(monitor, 25));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.synchronize.ResourceVariant#getUniquePath()
	 */
	public String getCachePath() {
		return getResourceUrl() + ' ' + getContentIdentifier();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.synchronize.ResourceVariant#getCacheId()
	 */
	protected String getCacheId() {
		return TeamWebDavPlugin.ID;
	}
	
}
